﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using SubSonicORM;
using System.Data.SqlClient;

namespace SubSonic.Tests
{
    [TestFixture]
    public class CustomerManagerTests
    {
        [SetUp]
        public void TestSetup()
        {
            DeleteCustomers();
        }

        [TearDown]
        public void TestTearDown()
        {
            DeleteCustomers();
        }

        private static void DeleteCustomers()
        {
            SqlConnection sqlConn = new SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["DatabaseConnection"].ConnectionString);
            SqlCommand sqlCmd = new SqlCommand("TRUNCATE TABLE dbo.Customer", sqlConn);

            try
            {
                sqlConn.Open();

                sqlCmd.ExecuteNonQuery();
            }
            finally
            {
                if (sqlConn.State == System.Data.ConnectionState.Open)
                    sqlConn.Close();
            }
        }


        [Test]
        public void CreateCustomer()
        {
            string name = "John Smith";
            string email = "JohnSmith@GoogleMail.com";
            Customer c = CustomerManager.CreateCustomer(name, email);
            Assert.AreEqual(name, c.Name);
            Assert.AreEqual(email, c.Email);
        }

        [Test]
        [ExpectedException(typeof(Exception))]
        public void CreateNullCustomer()
        {
            Customer c = CustomerManager.CreateCustomerWithoutValidation(null, null);
            Assert.Fail("Should not get here");
        }

        [Test]
        [ExpectedArgumentNullException]
        public void CreateNullCustomerWithOutValidation()
        {
            Customer c = CustomerManager.CreateCustomer(null, null);
            Assert.Fail("Should not get here");
        }

        [RowTest]
        [Row("John Smith")]
        [Row("Unknown Customer", ExpectedException = typeof(ArgumentException))]
        public void GetCustomer(string name)
        {
            CustomerManager.CreateCustomerWithoutValidation("John Smith", "");
            Customer c = CustomerManager.GetCustomer(name);
            Assert.AreEqual(name, c.Name);
        }

        [Test]
        [ExpectedArgumentException]
        public void GetCustomer()
        {
            string name = "John Smith";
            CustomerManager.CreateCustomerWithoutValidation(name, "");
            CustomerManager.CreateCustomerWithoutValidation(name, "");
            CustomerManager.CreateCustomerWithoutValidation(name, "");
            Customer c = CustomerManager.GetCustomer(name);
            Assert.AreEqual(name, c.Name);
        }

        [Test]
        public void GetAllCustomers()
        {
            string name = "John Smith";
            CustomerManager.CreateCustomerWithoutValidation(name + " A", "");
            CustomerManager.CreateCustomerWithoutValidation(name + " B", "");
            CustomerManager.CreateCustomerWithoutValidation(name + " C", "");
            List<Customer> customers = CustomerManager.GetAllCustomers();
            Assert.AreEqual(3, customers.Count);
        }

        [Test]
        public void GetCustomerID()
        {
            string name = "John Smith";
            CustomerManager.CreateCustomerWithoutValidation(name, "");

            Customer c = CustomerManager.GetCustomer(name);

            Assert.AreEqual(1, c.ID);
        }

        [Test]
        public void GetCustomerByID()
        {
            string name = "John Smith";
            CustomerManager.CreateCustomerWithoutValidation(name, "");

            Customer c = CustomerManager.GetCustomer(1);

            Assert.AreEqual(name, c.Name);
        }


        [Test]
        public void UpdateCustomer()
        {
            string name = "John Smith";
            CustomerManager.CreateCustomerWithoutValidation(name, "");
            Customer c = CustomerManager.GetCustomer(name);
            Assert.AreEqual(name, c.Name);

            c.Email = "JohnSmith@Gmail.com";

            CustomerManager.UpdateCustomer(c.ID, c);
        }
    }
}
